@@ -100,6 +100,8 @@ joinp options:
                            will be skipped. If not set, the query will fail.
                            Only use this when debugging queries, as polars does batched
                            parsing and will skip the entire batch where the error occurred.
+                           To get more detailed error messages, set the environment variable
+                           POLARS_BACKTRACE_IN_ERR=1 before running the join.
     --decimal-comma        Use comma as the decimal separator when parsing CSVs.
                            Otherwise, use period as the decimal separator.
                            Note that you'll need to set --delimiter to an alternate delimiter
@@ -191,7 +193,6 @@ use std::{
 
 use polars::{datatypes::AnyValue, prelude::*, sql::SQLContext};
 use serde::Deserialize;
-use smartstring::SmartString;
 use tempfile::tempdir;
 
 use crate::{cmd::sqlp::compress_output_if_needed, config::Delimiter, util, CliResult};
@@ -333,7 +334,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                     args.flag_left_by
                         .unwrap()
                         .split(',')
-                        .map(smartstring::SmartString::from)
+                        .map(PlSmallStr::from_str)
                         .collect(),
                 );
             }
@@ -342,7 +343,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                     args.flag_right_by
                         .unwrap()
                         .split(',')
-                        .map(smartstring::SmartString::from)
+                        .map(PlSmallStr::from_str)
                         .collect(),
                 );
             }
@@ -410,22 +411,33 @@ impl JoinStruct {
             JoinCoalesce::JoinSpecific
         };
 
-        let mut optimization_state = polars::lazy::frame::OptState::default();
-        if self.streaming {
-            optimization_state |= OptState::NEW_STREAMING;
-        }
+        let mut optflags = OptFlags::from_bits_truncate(0);
         if self.no_optimizations {
-            optimization_state = OptState::from_bits_truncate(0) | OptState::TYPE_COERCION;
+            optflags |= OptFlags::TYPE_COERCION;
+        } else {
+            optflags |= OptFlags::PROJECTION_PUSHDOWN
+                | OptFlags::PREDICATE_PUSHDOWN
+                | OptFlags::CLUSTER_WITH_COLUMNS
+                | OptFlags::TYPE_COERCION
+                | OptFlags::SIMPLIFY_EXPR
+                | OptFlags::FILE_CACHING
+                | OptFlags::SLICE_PUSHDOWN
+                | OptFlags::COMM_SUBPLAN_ELIM
+                | OptFlags::COMM_SUBEXPR_ELIM
+                | OptFlags::ROW_ESTIMATE
+                | OptFlags::FAST_PROJECTION;
         }
 
-        log::debug!("Optimization state: {optimization_state:?}");
+        optflags.set(OptFlags::STREAMING, self.streaming);
+
+        // log::debug!("Optimization flags: {optimization_flags:?}");
 
         let join_results = if jointype == JoinType::Cross {
             // cross join doesn't need join columns
             self.left_lf
-                .with_optimizations(optimization_state)
+                .with_optimizations(optflags)
                 .join_builder()
-                .with(self.right_lf.with_optimizations(optimization_state))
+                .with(self.right_lf.with_optimizations(optflags))
                 .how(JoinType::Cross)
                 .coalesce(coalesce_flag)
                 .allow_parallel(true)
@@ -435,25 +447,28 @@ impl JoinStruct {
         } else {
             if asof_join {
                 // sort by the asof columns, as asof joins require sorted join column data
-                let left_selcols_smartsvec: Vec<SmartString<smartstring::LazyCompact>> =
-                    self.left_sel.split(',').map(SmartString::from).collect();
+                let left_selcols_vec: Vec<PlSmallStr> =
+                    self.left_sel.split(',').map(PlSmallStr::from_str).collect();
 
                 self.left_lf = self
                     .left_lf
-                    .sort(&left_selcols_smartsvec, SortMultipleOptions::default());
+                    .sort(left_selcols_vec, SortMultipleOptions::default());
 
-                let right_selcols_smartsvec: Vec<SmartString<smartstring::LazyCompact>> =
-                    self.right_sel.split(',').map(SmartString::from).collect();
+                let right_selcols_vec: Vec<PlSmallStr> = self
+                    .right_sel
+                    .split(',')
+                    .map(PlSmallStr::from_str)
+                    .collect();
 
                 self.right_lf = self
                     .right_lf
-                    .sort(&right_selcols_smartsvec, SortMultipleOptions::default());
+                    .sort(right_selcols_vec, SortMultipleOptions::default());
             }
 
             self.left_lf
-                .with_optimizations(optimization_state)
+                .with_optimizations(optflags)
                 .join_builder()
-                .with(self.right_lf.with_optimizations(optimization_state))
+                .with(self.right_lf.with_optimizations(optflags))
                 .left_on(left_selcols)
                 .right_on(right_selcols)
                 .how(jointype)
@@ -521,7 +536,7 @@ impl Args {
         };
 
         let comment_char = if let Ok(comment_char) = env::var("QSV_COMMENT_CHAR") {
-            Some(comment_char)
+            Some(PlSmallStr::from_string(comment_char))
         } else {
             None
         };
@@ -554,7 +569,7 @@ impl Args {
             LazyCsvReader::new(&self.arg_input1)
                 .with_has_header(true)
                 .with_missing_is_null(self.flag_nulls)
-                .with_comment_prefix(comment_char.as_deref())
+                .with_comment_prefix(comment_char.clone())
                 .with_separator(tsvssv_delim(&self.arg_input1, delim))
                 .with_infer_schema_length(num_rows)
                 .with_try_parse_dates(try_parsedates)
@@ -580,7 +595,7 @@ impl Args {
             LazyCsvReader::new(&self.arg_input2)
                 .with_has_header(true)
                 .with_missing_is_null(self.flag_nulls)
-                .with_comment_prefix(comment_char.as_deref())
+                .with_comment_prefix(comment_char)
                 .with_separator(tsvssv_delim(&self.arg_input2, delim))
                 .with_infer_schema_length(num_rows)
                 .with_try_parse_dates(try_parsedates)
