@@ -52,11 +52,15 @@ pub mod joinp;
 pub mod json;
 #[cfg(any(feature = "feature_capable", feature = "lite"))]
 pub mod jsonl;
+#[cfg(feature = "lens")]
+pub mod lens;
 #[cfg(feature = "luau")]
 pub mod luau;
 #[cfg(any(feature = "feature_capable", feature = "lite"))]
 pub mod partition;
 #[cfg(any(feature = "feature_capable", feature = "lite"))]
+pub mod pro;
+#[cfg(any(feature = "feature_capable", feature = "lite"))]
 pub mod prompt;
 pub mod pseudo;
 #[cfg(all(feature = "python", feature = "feature_capable"))]
