@@ -1,16 +1,16 @@
 static USAGE: &str = r#"
 Compute summary statistics & infers data types for each column in a CSV. 
 
-Summary statistics includes sum, min/max/range, sort order, min/max length, mean, standard error of
-the mean (SEM), stddev, variance, coefficient of variation (CV), nullcount, max_precision, sparsity,
-quartiles, interquartile range (IQR), lower/upper fences, skewness, median, cardinality, mode/s &
-"antimode/s", and median absolute deviation (MAD). Note that some statistics require loading the
-entire file into memory, so they must be enabled explicitly. 
+Summary statistics includes sum, min/max/range, sort order, min/max/sum/avg length, mean,
+standard error of the mean (SEM), stddev, variance, coefficient of variation (CV), nullcount,
+max_precision, sparsity, quartiles, interquartile range (IQR), lower/upper fences, skewness, median,
+cardinality, mode/s & "antimode/s", and median absolute deviation (MAD). Note that some statistics
+require loading theentire file into memory, so they must be enabled explicitly. 
 
 By default, the following "streaming" statistics are reported for *every* column:
-sum, min/max/range values, sort order, min/max length, mean, sem, stddev, variance, cv, nullcount,
-max_precision and sparsity. The default set of statistics corresponds to ones that can be computed
-efficiently on a stream of data (i.e., constant memory) and works with arbitrarily large CSV files.
+sum, min/max/range values, sort order, min/max/sum/avg length, mean, sem, stddev, variance, cv,
+nullcount, max_precision & sparsity. The default set of statistics corresponds to ones that can be
+computed efficiently on a stream of data (i.e., constant memory) and works with arbitrarily large CSVs.
 
 The following additional "non-streaming" statistics require loading the entire file into memory:
 cardinality, mode/antimode, median, MAD, quartiles and its related measures (IQR,
@@ -114,6 +114,7 @@ Prompt for both INPUT and OUTPUT files in the ~/Documents dir with custom prompt
       qsv prompt -m 'Save summary to...' -d ~/Documents --fd-output --save-fname summarystats.csv
 
 For more examples, see https://github.com/jqnatividad/qsv/tree/master/resources/test
+For more info, see https://github.com/jqnatividad/qsv/wiki/Supplemental#stats-command-output-explanation
 
 Usage:
     qsv stats [options] [<input>]
@@ -248,6 +249,7 @@ use std::{
 use itertools::Itertools;
 use qsv_dateparser::parse_with_preference;
 use serde::{Deserialize, Serialize};
+use simd_json::{prelude::ValueAsScalar, OwnedValue};
 use simdutf8::basic::from_utf8;
 use stats::{merge_all, Commute, MinMax, OnlineStats, Unsorted};
 use tempfile::NamedTempFile;
@@ -319,6 +321,59 @@ struct StatsArgs {
     qsv_version:          String,
 }
 
+impl StatsArgs {
+    // this is for deserializing the stats.csv.jsonl file
+    fn from_owned_value(value: &OwnedValue) -> Result<Self, Box<dyn std::error::Error>> {
+        Ok(Self {
+            arg_input:            value["arg_input"].as_str().unwrap_or_default().to_string(),
+            flag_select:          value["flag_select"]
+                .as_str()
+                .unwrap_or_default()
+                .to_string(),
+            flag_everything:      value["flag_everything"].as_bool().unwrap_or_default(),
+            flag_typesonly:       value["flag_typesonly"].as_bool().unwrap_or_default(),
+            flag_infer_boolean:   value["flag_infer_boolean"].as_bool().unwrap_or_default(),
+            flag_mode:            value["flag_mode"].as_bool().unwrap_or_default(),
+            flag_cardinality:     value["flag_cardinality"].as_bool().unwrap_or_default(),
+            flag_median:          value["flag_median"].as_bool().unwrap_or_default(),
+            flag_mad:             value["flag_mad"].as_bool().unwrap_or_default(),
+            flag_quartiles:       value["flag_quartiles"].as_bool().unwrap_or_default(),
+            flag_round:           value["flag_round"].as_u64().unwrap_or_default() as u32,
+            flag_nulls:           value["flag_nulls"].as_bool().unwrap_or_default(),
+            flag_infer_dates:     value["flag_infer_dates"].as_bool().unwrap_or_default(),
+            flag_dates_whitelist: value["flag_dates_whitelist"]
+                .as_str()
+                .unwrap_or_default()
+                .to_string(),
+            flag_prefer_dmy:      value["flag_prefer_dmy"].as_bool().unwrap_or_default(),
+            flag_no_headers:      value["flag_no_headers"].as_bool().unwrap_or_default(),
+            flag_delimiter:       value["flag_delimiter"]
+                .as_str()
+                .unwrap_or_default()
+                .to_string(),
+            flag_output_snappy:   value["flag_output_snappy"].as_bool().unwrap_or_default(),
+            canonical_input_path: value["canonical_input_path"]
+                .as_str()
+                .unwrap_or_default()
+                .to_string(),
+            canonical_stats_path: value["canonical_stats_path"]
+                .as_str()
+                .unwrap_or_default()
+                .to_string(),
+            record_count:         value["record_count"].as_u64().unwrap_or_default(),
+            date_generated:       value["date_generated"]
+                .as_str()
+                .unwrap_or_default()
+                .to_string(),
+            compute_duration_ms:  value["compute_duration_ms"].as_u64().unwrap_or_default(),
+            qsv_version:          value["qsv_version"]
+                .as_str()
+                .unwrap_or_default()
+                .to_string(),
+        })
+    }
+}
+
 #[derive(Clone, Serialize, Deserialize, PartialEq, Default)]
 pub struct StatsData {
     pub field:                String,
@@ -334,6 +389,8 @@ pub struct StatsData {
     pub sort_order:           Option<String>,
     pub min_length:           Option<usize>,
     pub max_length:           Option<usize>,
+    pub sum_length:           Option<usize>,
+    pub avg_length:           Option<f64>,
     pub mean:                 Option<f64>,
     pub sem:                  Option<f64>,
     pub stddev:               Option<f64>,
@@ -382,6 +439,8 @@ pub static STATSDATA_TYPES_ARRAY: [JsonTypes; MAX_STAT_COLUMNS] = [
     JsonTypes::String, //sort_order
     JsonTypes::Int,    //min_length
     JsonTypes::Int,    //max_length
+    JsonTypes::Int,    //sum_length
+    JsonTypes::Float,  //avg_length
     JsonTypes::Float,  //mean
     JsonTypes::Float,  //sem
     JsonTypes::Float,  //stddev
@@ -412,6 +471,11 @@ pub static STATSDATA_TYPES_ARRAY: [JsonTypes; MAX_STAT_COLUMNS] = [
 static INFER_DATE_FLAGS: OnceLock<Vec<bool>> = OnceLock::new();
 static RECORD_COUNT: OnceLock<u64> = OnceLock::new();
 
+// standard overflow and underflow strings
+// for sum, sum_length and avg_length
+const OVERFLOW_STRING: &str = "*OVERFLOW*";
+const UNDERFLOW_STRING: &str = "*UNDERFLOW*";
+
 // number of milliseconds per day
 const MS_IN_DAY: f64 = 86_400_000.0;
 const MS_IN_DAY_INT: i64 = 86_400_000;
@@ -420,7 +484,7 @@ const MS_IN_DAY_INT: i64 = 86_400_000;
 const DAY_DECIMAL_PLACES: u32 = 5;
 
 // maximum number of output columns
-const MAX_STAT_COLUMNS: usize = 35;
+const MAX_STAT_COLUMNS: usize = 37;
 
 // maximum number of antimodes to display
 const MAX_ANTIMODES: usize = 10;
@@ -505,6 +569,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     };
     let stats_csv_tempfile_fname = format!(
         "{stem}.{prime_ext}{snappy_ext}",
+        //safety: we know the tempfile is a valid NamedTempFile, so we can use unwrap
         stem = stats_csv_tempfile.path().to_str().unwrap(),
         prime_ext = output_extension,
         snappy_ext = if snappy { ".sz" } else { "" }
@@ -547,6 +612,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     let write_stats_jsonl = args.flag_stats_jsonl;
 
     if let Some(path) = rconfig.path.clone() {
+        //safety: we know the path is a valid PathBuf, so we can use unwrap
         let path_file_stem = path.file_stem().unwrap().to_str().unwrap();
         let stats_file = stats_path(&path, false)?;
         // check if <FILESTEM>.stats.csv file already exists.
@@ -570,30 +636,48 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
             let time_saved: u64;
             // deserialize the existing stats args json
-            let existing_stats_args_json: StatsArgs =
-                match serde_json::from_str::<StatsArgs>(&existing_stats_args_json_str) {
-                    Ok(mut stat_args) => {
-                        // we init these fields to empty values because we don't want to compare
-                        // them when checking if the args are the same
-                        stat_args.canonical_input_path = String::new();
-                        stat_args.canonical_stats_path = String::new();
-                        stat_args.record_count = 0;
-                        stat_args.date_generated = String::new();
-                        time_saved = stat_args.compute_duration_ms;
-                        stat_args.compute_duration_ms = 0;
-                        stat_args
+            let existing_stats_args_json: StatsArgs = {
+                let mut json_buffer = existing_stats_args_json_str.into_bytes();
+                match simd_json::to_owned_value(&mut json_buffer) {
+                    Ok(value) => {
+                        // Convert OwnedValue to StatsArgs
+                        match StatsArgs::from_owned_value(&value) {
+                            Ok(mut stat_args) => {
+                                // we init these fields to empty values because we don't want to
+                                // compare them when checking if the
+                                // args are the same
+                                stat_args.canonical_input_path = String::new();
+                                stat_args.canonical_stats_path = String::new();
+                                stat_args.record_count = 0;
+                                stat_args.date_generated = String::new();
+                                time_saved = stat_args.compute_duration_ms;
+                                stat_args.compute_duration_ms = 0;
+                                stat_args
+                            },
+                            Err(e) => {
+                                time_saved = 0;
+                                log::warn!(
+                                    "Could not deserialize {path_file_stem}.stats.csv.json: \
+                                     {e:?}, recomputing..."
+                                );
+                                fs::remove_file(&stats_file)?;
+                                fs::remove_file(&stats_args_json_file)?;
+                                StatsArgs::default()
+                            },
+                        }
                     },
                     Err(e) => {
                         time_saved = 0;
                         log::warn!(
-                            "Could not serialize {path_file_stem}.stats.csv.json: {e:?}, \
+                            "Could not parse {path_file_stem}.stats.csv.json: {e:?}, \
                              recomputing..."
                         );
                         fs::remove_file(&stats_file)?;
                         fs::remove_file(&stats_args_json_file)?;
                         StatsArgs::default()
                     },
-                };
+                }
+            };
 
             // check if the cached stats are current (ie the stats file is newer than the input
             // file), use the same args or if the --everything flag was set, and
@@ -654,8 +738,9 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             }
 
             // we need to count the number of records in the file to calculate sparsity
+            // safety: we know util::count_rows() will not return an Err, so we can use unwrap
             let record_count = RECORD_COUNT.get_or_init(|| util::count_rows(&rconfig).unwrap());
-            log::info!("scanning {record_count} records...");
+            // log::info!("scanning {record_count} records...");
 
             let (headers, stats) = match rconfig.indexed()? {
                 None => args.sequential_stats(&args.flag_dates_whitelist),
@@ -696,6 +781,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             {
                 // if the stats run took longer than the cache threshold and the threshold > 0,
                 // cache the stats so we don't have to recompute it next time
+                // safety: we know the path is a valid PathBuf, so we can use unwrap
                 current_stats_args.canonical_input_path =
                     path.canonicalize()?.to_str().unwrap().to_string();
                 current_stats_args.record_count = *record_count;
@@ -720,6 +806,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         stats_csv_tempfile_fname
     } else {
         // we didn't compute the stats, re-use the existing stats file
+        // safety: we know the path is a valid PathBuf, so we can use unwrap
         stats_path(rconfig.path.as_ref().unwrap(), false)?
             .to_str()
             .unwrap()
@@ -728,6 +815,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
     if rconfig.is_stdin() {
         // if we read from stdin, copy the temp stats file to "stdin.stats.csv"
+        // safety: we know the path is a valid PathBuf, so we can use unwrap
         let mut stats_pathbuf = stats_path(rconfig.path.as_ref().unwrap(), true)?;
         fs::copy(currstats_filename.clone(), stats_pathbuf.clone())?;
 
@@ -735,12 +823,13 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         stats_pathbuf.set_extension("csv.json");
         std::fs::write(
             stats_pathbuf,
-            serde_json::to_string_pretty(&current_stats_args).unwrap(),
+            serde_json::to_string_pretty(&current_stats_args)?,
         )?;
     } else if let Some(path) = rconfig.path {
         // if we read from a file, copy the temp stats file to "<FILESTEM>.stats.csv"
         let mut stats_pathbuf = path.clone();
         stats_pathbuf.set_extension("stats.csv");
+        // safety: we know the path is a valid PathBuf, so we can use unwrap
         if currstats_filename != stats_pathbuf.to_str().unwrap() {
             // if the stats file is not the same as the input file, copy it
             fs::copy(currstats_filename.clone(), stats_pathbuf.clone())?;
@@ -777,6 +866,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             stats_pathbuf.set_extension("csv.json");
             // write empty file first so we can canonicalize it
             std::fs::File::create(stats_pathbuf.clone())?;
+            // safety: we know the path is a valid PathBuf, so we can use unwrap
             current_stats_args.canonical_stats_path = stats_pathbuf
                 .clone()
                 .canonicalize()?
@@ -785,8 +875,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                 .to_string();
             std::fs::write(
                 stats_pathbuf.clone(),
-                // safety: we know that current_stats_args is JSON serializable
-                serde_json::to_string_pretty(&current_stats_args).unwrap(),
+                serde_json::to_string_pretty(&current_stats_args)?,
             )?;
 
             // save the stats data to "<FILESTEM>.stats.csv.data.jsonl"
@@ -897,7 +986,8 @@ impl Args {
     where
         I: Iterator<Item = csv::Result<csv::ByteRecord>>,
     {
-        let mut stats = self.new_stats(sel.len());
+        let sel_len = sel.len();
+        let mut stats = self.new_stats(sel_len);
 
         // safety: we know INFER_DATE_FLAGS is Some because we called init_date_inference
         let infer_date_flags = INFER_DATE_FLAGS.get().unwrap();
@@ -908,12 +998,15 @@ impl Args {
         let prefer_dmy = self.flag_prefer_dmy;
 
         let mut i;
+        #[allow(unused_assignments)]
+        let mut current_row = csv::ByteRecord::with_capacity(1024, sel_len);
         for row in it {
             i = 0;
             // safety: because we're using iterators and INFER_DATE_FLAGS has the same size,
             // we know we don't need to bounds check
             unsafe {
-                for field in sel.select(&row.unwrap_unchecked()) {
+                current_row = row.unwrap_unchecked();
+                for field in sel.select(&current_row) {
                     stats.get_unchecked_mut(i).add(
                         field,
                         *infer_date_flags.get_unchecked(i),
@@ -971,8 +1064,11 @@ impl Args {
             return csv::StringRecord::from(vec!["field", "type"]);
         }
 
-        // with --everything, we have 35 columns at most
+        // with --everything, we have MAX_STAT_COLUMNS columns at most
         let mut fields = Vec::with_capacity(MAX_STAT_COLUMNS);
+
+        // these are the standard stats columns that are always output
+        // the "streaming" stats
         fields.extend_from_slice(&[
             "field",
             "type",
@@ -984,6 +1080,8 @@ impl Args {
             "sort_order",
             "min_length",
             "max_length",
+            "sum_length",
+            "avg_length",
             "mean",
             "sem",
             "stddev",
@@ -993,6 +1091,8 @@ impl Args {
             "max_precision",
             "sparsity",
         ]);
+
+        // these are the stats columns that are only output if the user requested them
         let all = self.flag_everything;
         if self.flag_median && !self.flag_quartiles && !all {
             fields.push("median");
@@ -1078,6 +1178,7 @@ fn init_date_inference(
         headers
             .iter()
             .map(|header| {
+                // safety: we know the header is a valid String, so we can use unwrap
                 util::to_lowercase_into(&from_bytes::<String>(header).unwrap(), &mut header_str);
                 date_found = whitelist
                     .iter()
@@ -1122,6 +1223,7 @@ pub struct Stats {
     typ:           FieldType,
     is_ascii:      bool,
     sum:           Option<TypedSum>,
+    sum_stotlen:   u64,
     minmax:        Option<TypedMinMax>,
     online:        Option<OnlineStats>,
     nullcount:     u64,
@@ -1175,6 +1277,7 @@ impl Stats {
             typ: FieldType::default(),
             is_ascii: true,
             sum,
+            sum_stotlen: 0,
             minmax,
             online,
             nullcount: 0,
@@ -1237,6 +1340,7 @@ impl Stats {
                         };
                     }
                 } else {
+                    // safety: we know the sample is a valid f64, so we can use unwrap
                     let n = from_bytes::<f64>(sample).unwrap();
                     if let Some(v) = self.median.as_mut() {
                         v.add(n);
@@ -1306,15 +1410,42 @@ impl Stats {
 
         let typ = self.typ;
         // prealloc memory for performance
-        // we have 35 columns at most with --everything
+        // we have MAX_STAT_COLUMNS columns at most with --everything
         let mut pieces = Vec::with_capacity(MAX_STAT_COLUMNS);
 
         let empty = String::new;
 
-        // mode/modes & cardinality
-        // we do this first because we need to know the cardinality to --infer-boolean
-        // should that be enabled
-        let mut cardinality = 0_usize;
+        // min/max/range/sort_order
+        // we do this first as we want to get the sort_order, so we can skip sorting if not
+        // required. We also need to do this before --infer-boolean because we need to know
+        // the min/max values to determine if the range is equal to the supported boolean
+        // ranges (0/1, f/t, n/y)
+        let mut minmax_range_sortorder_pieces = Vec::with_capacity(4);
+        let mut minval_lower: char = '\0';
+        let mut maxval_lower: char = '\0';
+        let mut column_sorted = false;
+        if let Some(mm) = self
+            .minmax
+            .as_ref()
+            .and_then(|mm| mm.show(typ, round_places))
+        {
+            // get first character of min/max values
+            minval_lower = mm.0.chars().next().unwrap_or_default().to_ascii_lowercase();
+            maxval_lower = mm.1.chars().next().unwrap_or_default().to_ascii_lowercase();
+            if mm.3.starts_with("Ascending") {
+                column_sorted = true;
+            }
+            minmax_range_sortorder_pieces.extend_from_slice(&[mm.0, mm.1, mm.2, mm.3]);
+        } else {
+            minmax_range_sortorder_pieces.extend_from_slice(&[empty(), empty(), empty(), empty()]);
+        }
+
+        // modes/antimodes & cardinality
+        // we do this second because we can use the sort order with cardinality, to skip sorting
+        // if its not required. This makes not only cardinality computation faster, it also makes
+        // modes/antimodes computation faster.
+        // We also need to know the cardinality to --infer-boolean should that be enabled
+        let mut cardinality = 0;
         let mut mc_pieces = Vec::with_capacity(7);
         match self.modes.as_mut() {
             None => {
@@ -1327,7 +1458,7 @@ impl Stats {
             },
             Some(ref mut v) => {
                 if self.which.cardinality {
-                    cardinality = v.cardinality();
+                    cardinality = v.cardinality(column_sorted, 1);
                     let mut buffer = itoa::Buffer::new();
                     mc_pieces.push(buffer.format(cardinality).to_owned());
                 }
@@ -1389,25 +1520,6 @@ impl Stats {
             },
         }
 
-        // min/max/range/sort_order
-        // we also do this before --infer-boolean because we need to know the min/max values
-        // to determine if the range is equal to the supported boolean ranges (0/1, f/t, n/y)
-        let mut minmax_range_sortorder_pieces = Vec::with_capacity(4);
-        let mut minval_lower: char = '\0';
-        let mut maxval_lower: char = '\0';
-        if let Some(mm) = self
-            .minmax
-            .as_ref()
-            .and_then(|mm| mm.show(typ, round_places))
-        {
-            // get first character of min/max values
-            minval_lower = mm.0.chars().next().unwrap_or_default().to_ascii_lowercase();
-            maxval_lower = mm.1.chars().next().unwrap_or_default().to_ascii_lowercase();
-            minmax_range_sortorder_pieces.extend_from_slice(&[mm.0, mm.1, mm.2, mm.3]);
-        } else {
-            minmax_range_sortorder_pieces.extend_from_slice(&[empty(), empty(), empty(), empty()]);
-        }
-
         // type
         if cardinality == 2 && infer_boolean {
             // if cardinality is 2, it's a boolean if its values' first character are 0/1, f/t, n/y
@@ -1436,33 +1548,57 @@ impl Stats {
         }
 
         // sum
-        if let Some(sum) = self.sum.as_ref().and_then(|sum| sum.show(typ)) {
-            if typ == FieldType::TFloat {
-                if let Ok(f64_val) = sum.parse::<f64>() {
-                    pieces.push(util::round_num(f64_val, round_places));
+        let stotlen =
+            if let Some((stotlen_work, sum)) = self.sum.as_ref().and_then(|sum| sum.show(typ)) {
+                if typ == FieldType::TFloat {
+                    if let Ok(f64_val) = sum.parse::<f64>() {
+                        pieces.push(util::round_num(f64_val, round_places));
+                    } else {
+                        pieces.push(format!("ERROR: Cannot convert {sum} to a float."));
+                    }
                 } else {
-                    pieces.push(format!("ERROR: Cannot convert {sum} to a float."));
+                    pieces.push(sum);
                 }
+                stotlen_work
             } else {
-                pieces.push(sum);
-            }
-        } else {
-            pieces.push(empty());
-        }
+                pieces.push(empty());
+                0
+            };
 
         // min/max/range/sort_order
         // actually append it here - to preserve legacy ordering of columns
         pieces.extend_from_slice(&minmax_range_sortorder_pieces);
 
-        // min/max length
+        // min/max/sum/avg length
         if typ == FieldType::TDate || typ == FieldType::TDateTime {
             // returning min/max length for dates doesn't make sense
             // especially since we convert the date stats to rfc3339 format
-            pieces.extend_from_slice(&[empty(), empty()]);
+            pieces.extend_from_slice(&[empty(), empty(), empty(), empty()]);
         } else if let Some(mm) = self.minmax.as_ref().and_then(TypedMinMax::len_range) {
             pieces.extend_from_slice(&[mm.0, mm.1]);
+            // we have a sum_length
+            if stotlen > 0 {
+                if stotlen < u64::MAX {
+                    // so we can compute avg_length
+                    let mut buffer = itoa::Buffer::new();
+                    pieces.push(buffer.format(stotlen).to_owned());
+                    pieces.push(util::round_num(
+                        stotlen as f64 / *RECORD_COUNT.get().unwrap_or(&1) as f64,
+                        4,
+                    ));
+                } else {
+                    // however, we saturated the sum, it means we had an overflow
+                    // so we return OVERFLOW_STRING for sum and avg length
+                    pieces.extend_from_slice(&[
+                        OVERFLOW_STRING.to_string(),
+                        OVERFLOW_STRING.to_string(),
+                    ]);
+                }
+            } else {
+                pieces.extend_from_slice(&[empty(), empty()]);
+            }
         } else {
-            pieces.extend_from_slice(&[empty(), empty()]);
+            pieces.extend_from_slice(&[empty(), empty(), empty(), empty()]);
         }
 
         // mean, sem, stddev, variance & cv
@@ -1518,10 +1654,6 @@ impl Stats {
         }
 
         // sparsity
-        // stats is also called by the `schema` and `tojsonl` commands to infer a schema,
-        // sparsity is not required by those cmds and we don't necessarily have a valid
-        // RECORD_COUNT when called by those cmds, so just set sparsity to nullcount
-        // (i.e. divide by 1) so we don't panic.
         #[allow(clippy::cast_precision_loss)]
         let sparsity: f64 = self.nullcount as f64 / *RECORD_COUNT.get().unwrap_or(&1) as f64;
         pieces.push(util::round_num(sparsity, round_places));
@@ -1661,6 +1793,7 @@ impl Commute for Stats {
         self.typ.merge(other.typ);
         self.is_ascii &= other.is_ascii;
         self.sum.merge(other.sum);
+        self.sum_stotlen = self.sum_stotlen.saturating_add(other.sum_stotlen);
         self.minmax.merge(other.minmax);
         self.online.merge(other.online);
         self.nullcount += other.nullcount;
@@ -1804,9 +1937,11 @@ impl fmt::Debug for FieldType {
 
 /// `TypedSum` keeps a rolling sum of the data seen.
 /// It sums integers until it sees a float, at which point it sums floats.
+/// It also counts the total length of strings.
 #[derive(Clone, Default, Serialize, Deserialize, PartialEq)]
 struct TypedSum {
     integer: i64,
+    stotlen: u64, // sum of the total length of strings
     float:   Option<f64>,
 }
 
@@ -1817,8 +1952,10 @@ impl TypedSum {
             return;
         }
         #[allow(clippy::cast_precision_loss)]
+        // safety: we know the sample is valid for their respective types, so we can use unwrap
         match typ {
             TFloat => {
+                self.stotlen = self.stotlen.saturating_add(sample.len() as u64);
                 let float: f64 = from_bytes::<f64>(sample).unwrap();
                 match self.float {
                     None => {
@@ -1830,6 +1967,7 @@ impl TypedSum {
                 }
             },
             TInteger => {
+                self.stotlen = self.stotlen.saturating_add(sample.len() as u64);
                 if let Some(ref mut float) = self.float {
                     *float += from_bytes::<f64>(sample).unwrap();
                 } else {
@@ -1839,29 +1977,36 @@ impl TypedSum {
                         .saturating_add(atoi_simd::parse::<i64>(sample).unwrap());
                 }
             },
+            TString => {
+                self.stotlen = self.stotlen.saturating_add(sample.len() as u64);
+            },
             _ => {},
         }
     }
 
-    fn show(&self, typ: FieldType) -> Option<String> {
+    fn show(&self, typ: FieldType) -> Option<(u64, String)> {
         match typ {
-            TNull | TString | TDate | TDateTime => None,
+            TNull | TDate | TDateTime => None,
             TInteger => {
                 match self.integer {
                     // with saturating_add, if this is equal to i64::MAX or i64::MIN
                     // we overflowed/underflowed
-                    i64::MAX => Some("OVERFLOW".to_string()),
-                    i64::MIN => Some("UNDERFLOW".to_string()),
+                    i64::MAX => Some((self.stotlen, OVERFLOW_STRING.to_string())),
+                    i64::MIN => Some((self.stotlen, UNDERFLOW_STRING.to_string())),
                     _ => {
                         let mut buffer = itoa::Buffer::new();
-                        Some(buffer.format(self.integer).to_owned())
+                        Some((self.stotlen, buffer.format(self.integer).to_owned()))
                     },
                 }
             },
             TFloat => {
-                let mut buffer = ryu::Buffer::new();
-                Some(buffer.format(self.float.unwrap_or(0.0)).to_owned())
+                let mut fbuffer = ryu::Buffer::new();
+                Some((
+                    self.stotlen,
+                    fbuffer.format(self.float.unwrap_or(0.0)).to_owned(),
+                ))
             },
+            TString => Some((self.stotlen, String::new())),
         }
     }
 }
@@ -1876,6 +2021,7 @@ impl Commute for TypedSum {
             (None, Some(f2)) => self.float = Some((self.integer as f64) + f2),
             (None, None) => self.integer = self.integer.saturating_add(other.integer),
         }
+        self.stotlen = self.stotlen.saturating_add(other.stotlen);
     }
 }
 
