@@ -147,11 +147,15 @@ fn main() -> QsvExitCode {
     jsonl       Convert newline-delimited JSON files to CSV\n",
     );
 
+    #[cfg(all(feature = "lens", feature = "feature_capable"))]
+    enabled_commands.push_str("    lens        View a CSV file interactively\n");
+
     #[cfg(all(feature = "luau", feature = "feature_capable"))]
     enabled_commands.push_str("    luau        Execute Luau script on CSV data\n");
 
     enabled_commands.push_str(
         "    partition   Partition CSV data based on a column value
+    pro         Interact with the qsv pro API
     prompt      Open a file dialog to pick a file
     pseudo      Pseudonymise the values of a column\n",
     );
@@ -362,9 +366,12 @@ enum Command {
     JoinP,
     Json,
     Jsonl,
+    #[cfg(all(feature = "lens", feature = "feature_capable"))]
+    Lens,
     #[cfg(all(feature = "luau", feature = "feature_capable"))]
     Luau,
     Partition,
+    Pro,
     Prompt,
     Pseudo,
     #[cfg(all(feature = "python", feature = "feature_capable"))]
@@ -454,9 +461,12 @@ impl Command {
             Command::JoinP => cmd::joinp::run(argv),
             Command::Json => cmd::json::run(argv),
             Command::Jsonl => cmd::jsonl::run(argv),
+            #[cfg(all(feature = "lens", feature = "feature_capable"))]
+            Command::Lens => cmd::lens::run(argv),
             #[cfg(all(feature = "luau", feature = "feature_capable"))]
             Command::Luau => cmd::luau::run(argv),
             Command::Partition => cmd::partition::run(argv),
+            Command::Pro => cmd::pro::run(argv),
             Command::Prompt => cmd::prompt::run(argv),
             Command::Pseudo => cmd::pseudo::run(argv),
             #[cfg(all(feature = "python", feature = "feature_capable"))]
