@@ -33,7 +33,7 @@ macro_rules! joinp_test_tab {
             fn headers() {
                 let wrk = setup(stringify!($name0));
                 let mut cmd = wrk.command("joinp");
-                cmd.args(&["city", "cities.tsv", "city", "places.tab"]);
+                cmd.args(&["city", "cities.tsv", "city", "places.ssv"]);
                 $fun(wrk, cmd);
             }
         }
@@ -54,7 +54,7 @@ macro_rules! joinp_test_comments {
                 let wrk = setup(stringify!($name2));
                 let mut cmd = wrk.command("joinp");
                 cmd.env("QSV_COMMENT_CHAR", "#");
-                cmd.args(&["city", "cities_comments.csv", "city", "places.csv"]);
+                cmd.args(&["city", "cities_comments.csv", "city", "places.ssv"]);
                 $fun(wrk, cmd);
             }
         }
@@ -74,7 +74,7 @@ macro_rules! joinp_test_compressed {
             fn headers() {
                 let wrk = setup(stringify!($name3));
                 let mut cmd = wrk.command("joinp");
-                cmd.args(&["city", "cities.csv.sz", "city", "places.csv.sz"]);
+                cmd.args(&["city", "cities.csv.sz", "city", "places.ssv.sz"]);
                 $fun(wrk, cmd);
             }
         }
@@ -111,9 +111,9 @@ fn setup(name: &str) -> Workdir {
     wrk.create("cities_comments.csv", cities_comments);
     wrk.create("places.csv", places.clone());
 
-    // create TSV/TAB versions
+    // create TSV/SSV versions
     wrk.create_with_delim("cities.tsv", cities, b'\t');
-    wrk.create_with_delim("places.tab", places, b'\t');
+    wrk.create_with_delim("places.ssv", places.clone(), b';');
 
     // create snappy compressed versions
     let out_file = wrk.path("cities.csv.sz").to_string_lossy().to_string();
@@ -131,6 +131,13 @@ fn setup(name: &str) -> Workdir {
         .args(["--output", &out_file2]);
     wrk.assert_success(&mut cmd2);
 
+    let out_file3 = wrk.path("places.ssv.sz").to_string_lossy().to_string();
+    let mut cmd3 = wrk.command("snappy");
+    cmd3.arg("compress")
+        .arg("places.ssv")
+        .args(["--output", &out_file3]);
+    wrk.assert_success(&mut cmd3);
+
     wrk
 }
 
@@ -350,6 +357,45 @@ joinp_test!(
     }
 );
 
+joinp_test!(
+    joinp_outer_left_validate_none_streaming,
+    |wrk: Workdir, mut cmd: process::Command| {
+        cmd.arg("--left")
+            .args(["--validate", "none"])
+            .arg("--streaming");
+        let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+        let expected = make_rows(
+            false,
+            vec![
+                svec!["Boston", "MA", "Logan Airport"],
+                svec!["Boston", "MA", "Boston Garden"],
+                svec!["New York", "NY", ""],
+                svec!["San Francisco", "CA", ""],
+                svec!["Buffalo", "NY", "Ralph Wilson Stadium"],
+            ],
+        );
+        assert_eq!(got, expected);
+    }
+);
+
+joinp_test!(
+    joinp_outer_right_none_streaming,
+    |wrk: Workdir, mut cmd: process::Command| {
+        cmd.arg("--right")
+            .args(["--validate", "none"])
+            .arg("--streaming");
+        let got: Vec<Vec<String>> = wrk.read_stdout(&mut cmd);
+        let expected: Vec<Vec<String>> = vec![
+            svec!["state", "city", "place"],
+            svec!["MA", "Boston", "Logan Airport"],
+            svec!["MA", "Boston", "Boston Garden"],
+            svec!["NY", "Buffalo", "Ralph Wilson Stadium"],
+            svec!["", "Orlando", "Disney World"],
+        ];
+        assert_eq!(got, expected);
+    }
+);
+
 joinp_test_comments!(
     joinp_outer_left_validate_none_comments,
     |wrk: Workdir, mut cmd: process::Command| {
@@ -376,8 +422,20 @@ joinp_test!(
         let got: String = wrk.output_stderr(&mut cmd);
         assert_eq!(
             got,
-            "Polars error: ComputeError(ErrString(\"the join keys did not fulfil m:1 \
-             validation\"))\n"
+            "Polars error: ComputeError(ErrString(\"join keys did not fulfill m:1 validation\"))\n"
+        );
+        wrk.assert_err(&mut cmd);
+    }
+);
+
+joinp_test!(
+    joinp_outer_invalid_validation,
+    |wrk: Workdir, mut cmd: process::Command| {
+        cmd.arg("--left").args(["--validate", "manytoeveryone"]);
+        let got: String = wrk.output_stderr(&mut cmd);
+        assert_eq!(
+            got,
+            "usage error: Invalid join validation: manytoeveryone\n"
         );
         wrk.assert_err(&mut cmd);
     }
@@ -400,8 +458,7 @@ joinp_test!(
         let got: String = wrk.output_stderr(&mut cmd);
         assert_eq!(
             got,
-            "Polars error: ComputeError(ErrString(\"the join keys did not fulfil 1:1 \
-             validation\"))\n"
+            "Polars error: ComputeError(ErrString(\"join keys did not fulfill 1:1 validation\"))\n"
         );
         wrk.assert_err(&mut cmd);
     }
